var path_1 = require('path');
var repl_1 = require('repl');
var util_1 = require('util');
var Module = require('module');
var minimist = require('minimist');
var typescript_node_1 = require('../typescript-node');
var argv = minimist(process.argv.slice(2), {
    stopEarly: true,
    string: ['eval', 'print', 'compiler', 'configFile', 'ignoreWarnings'],
    boolean: ['help', 'version'],
    alias: {
        v: ['version'],
        e: ['eval'],
        p: ['print'],
        c: ['compiler'],
        f: ['configFile'],
        i: ['ignoreWarnings']
    }
});
if (argv.version) {
    console.log(typescript_node_1.VERSION);
    process.exit(0);
}
if (argv.help) {
    console.log("\n  Usage: ts-node [options] [ -e script | script.ts ] [arguments]\n\n  Options:\n\n    -e, --eval [code]             Evaluate code\n    -p, --print [code]            Evaluate code and print result\n    -c, --compiler [name]         Specify a custom TypeScript compiler\n    -i, --ignoreWarnings [codes]  Ignore TypeScript warnings by code\n    -f, --configFile [path]       Specify the path to `tsconfig.json`\n");
    process.exit(0);
}
var cwd = process.cwd();
var code = argv.eval == null ? argv.print : argv.eval;
var isEval = typeof code === 'string' || argv._.length === 0;
var compile = typescript_node_1.register({
    getFile: isEval ? getFileEval : typescript_node_1.getFile,
    getVersion: isEval ? getVersionEval : typescript_node_1.getVersion,
    compiler: argv.compiler,
    ignoreWarnings: list(argv.ignoreWarnings),
    configFile: argv.configFile,
    isEval: isEval
});
var EVAL_FILENAME = '[eval].ts';
var EVAL_PATH = path_1.join(cwd, EVAL_FILENAME);
var evalFile = { text: '', version: 0 };
if (typeof code === 'string') {
    global.__filename = EVAL_FILENAME;
    global.__dirname = cwd;
    var module_1 = new Module(global.__filename);
    module_1.filename = global.__filename;
    module_1.paths = Module._nodeModulePaths(global.__dirname);
    global.exports = module_1.exports;
    global.module = module_1;
    global.require = module_1.require.bind(module_1);
    var result;
    try {
        result = _eval(code);
    }
    catch (err) {
        if (err instanceof typescript_node_1.TypeScriptError) {
            console.error(err.message);
            process.exit(1);
        }
        throw err;
    }
    if (argv.print != null) {
        console.log(typeof result === 'string' ? result : util_1.inspect(result));
    }
}
else {
    if (argv._.length) {
        var args = argv._.slice();
        args[0] = path_1.join(cwd, args[0]);
        process.argv = ['node'].concat(args);
        process.execArgv.unshift(__filename);
        Module.runMain();
    }
    else {
        startRepl();
    }
}
function _eval(code) {
    evalFile.text = code;
    evalFile.version++;
    return (0, eval)(compile(EVAL_PATH));
}
function startRepl() {
    return repl_1.start({
        prompt: '> ',
        input: process.stdin,
        output: process.stdout,
        eval: replEval,
        useGlobal: true
    });
}
function replEval(code, context, filename, callback) {
    var err;
    var result;
    try {
        result = _eval(code);
    }
    catch (e) {
        if (e instanceof typescript_node_1.TypeScriptError) {
            err = e.message;
        }
        else {
            err = e;
        }
    }
    callback(err, result);
}
function list(value) {
    return String(value).split(/ *, */);
}
function getFileEval(fileName) {
    return fileName === EVAL_PATH ? evalFile.text : typescript_node_1.getFile(fileName);
}
function getVersionEval(fileName) {
    return fileName === EVAL_PATH ? String(evalFile.version) : typescript_node_1.getVersion(fileName);
}
//# sourceMappingURL=ts-node.js.map